@@ -1,5 +1,5 @@
 /*
- * swigutil_py.c: utility functions for the SWIG Perl bindings
+ * swigutil_pl.c: utility functions for the SWIG Perl bindings
  *
  * ====================================================================
  * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
@@ -21,6 +21,9 @@
 #include <XSUB.h>
 
 #include <stdarg.h>
+#ifdef WIN32
+#include <io.h>
+#endif
 
 #include <apr.h>
 #include <apr_general.h>
@@ -29,14 +32,7 @@
 #include "svn_pools.h"
 #include "svn_opt.h"
 
-#if SVN_SWIG_VERSION >= 103024
-#if SVN_SWIG_VERSION >= 103025
-#include <swiglabels.swg>
-#endif
-#include <swigrun.swg>
-#include <perl5/perlrun.swg>
-#include <runtime.swg>
-#endif
+#include "swig_perl_external_runtime.swg"
 
 #include "swigutil_pl.h"
 
@@ -1377,7 +1373,12 @@ apr_file_t *svn_swig_pl_make_file (SV *file, apr_pool_t *pool)
                     pool);
     } else if (SvROK(file) && SvTYPE(SvRV(file)) == SVt_PVGV) {
         apr_status_t status;
+#ifdef WIN32
+        apr_os_file_t osfile = (apr_os_file_t)
+          _get_osfhandle(PerlIO_fileno(IoIFP(sv_2io(file))));
+#else
         apr_os_file_t osfile = PerlIO_fileno(IoIFP(sv_2io(file)));
+#endif
         status = apr_os_file_put (&apr_file, &osfile, 
                                   O_CREAT | O_WRONLY, pool);
         if (status)
@@ -1398,3 +1399,14 @@ void svn_swig_pl_hold_ref_in_pool (apr_pool_t *pool, SV *sv)
     SvREFCNT_inc(sv);
     apr_pool_cleanup_register (pool, sv, cleanup_refcnt, apr_pool_cleanup_null);
 }
+
+SV *svn_swig_pl_from_md5 (unsigned char *digest)
+{
+    SV *ret;
+
+    svn_swig_pl_callback_thunk (CALL_METHOD, (void *)"new", &ret, "sS",
+			        "SVN::MD5", digest,
+                                _SWIG_TYPE("unsigned char *"));
+
+    return sv_2mortal (ret);
+}
