@@ -1,7 +1,7 @@
 /**
  * @copyright
  * ====================================================================
- * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2005 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -51,6 +51,8 @@
 extern "C" {
 #endif /* __cplusplus */
 
+#ifndef DOXYGEN_SHOULD_SKIP_THIS
+
 #if defined(SVN_ERROR_BUILD_ARRAY)
 
 #define SVN_ERROR_START \
@@ -92,8 +94,6 @@ extern "C" {
         perturbed.
 */
 
-#ifndef DOXYGEN_SHOULD_SKIP_THIS
-
 #define SVN_ERR_CATEGORY_SIZE 5000
 
 /* Leave one category of room at the beginning, for SVN_WARNING and
@@ -141,6 +141,8 @@ extern "C" {
 
 #endif /* DOXYGEN_SHOULD_SKIP_THIS */
 
+/** Collection of Subversion error code values, located within the
+ * APR user error space. */
 SVN_ERROR_START
 
   /* validation ("BAD_FOO") errors */
@@ -189,6 +191,10 @@ SVN_ERROR_START
               SVN_ERR_XML_CATEGORY_START + 3,
               "XML data was not well-formed")
 
+  SVN_ERRDEF (SVN_ERR_XML_UNESCAPABLE_DATA,
+              SVN_ERR_XML_CATEGORY_START + 4,
+              "Data cannot be safely XML-escaped")
+
   /* io errors */
 
   SVN_ERRDEF (SVN_ERR_IO_INCONSISTENT_EOL,
@@ -199,6 +205,7 @@ SVN_ERROR_START
               SVN_ERR_IO_CATEGORY_START + 1,
               "Unrecognized line ending style")
 
+  /** @deprecated Unused, slated for removal in the next major release. */
   SVN_ERRDEF (SVN_ERR_IO_CORRUPT_EOL,
               SVN_ERR_IO_CATEGORY_START + 2,
               "Line endings other than expected")
@@ -207,14 +214,20 @@ SVN_ERROR_START
               SVN_ERR_IO_CATEGORY_START + 3,
               "Ran out of unique names")
 
+  /** @deprecated Unused, slated for removal in the next major release. */
   SVN_ERRDEF (SVN_ERR_IO_PIPE_FRAME_ERROR,
               SVN_ERR_IO_CATEGORY_START + 4,
               "Framing error in pipe protocol")
 
+  /** @deprecated Unused, slated for removal in the next major release. */
   SVN_ERRDEF (SVN_ERR_IO_PIPE_READ_ERROR,
               SVN_ERR_IO_CATEGORY_START + 5,
               "Read error in pipe")
 
+  SVN_ERRDEF (SVN_ERR_IO_WRITE_ERROR,
+              SVN_ERR_IO_CATEGORY_START + 6,
+              "Write error")
+
   /* stream errors */
 
   SVN_ERRDEF (SVN_ERR_STREAM_UNEXPECTED_EOF,
@@ -269,14 +282,17 @@ SVN_ERROR_START
               SVN_ERR_WC_CATEGORY_START + 0,
               "Obstructed update")
 
+  /** @deprecated Unused, slated for removal in the next major release. */
   SVN_ERRDEF (SVN_ERR_WC_UNWIND_MISMATCH,
               SVN_ERR_WC_CATEGORY_START + 1,
               "Mismatch popping the WC unwind stack")
 
+  /** @deprecated Unused, slated for removal in the next major release. */
   SVN_ERRDEF (SVN_ERR_WC_UNWIND_EMPTY,
               SVN_ERR_WC_CATEGORY_START + 2,
               "Attempt to pop empty WC unwind stack")
 
+  /** @deprecated Unused, slated for removal in the next major release. */
   SVN_ERRDEF (SVN_ERR_WC_UNWIND_NOT_EMPTY,
               SVN_ERR_WC_CATEGORY_START + 3,
               "Attempt to unlock with non-empty unwind stack")
@@ -287,8 +303,9 @@ SVN_ERROR_START
 
   SVN_ERRDEF (SVN_ERR_WC_NOT_LOCKED,
               SVN_ERR_WC_CATEGORY_START + 5,
-              "Working copy not locked")
+              "Working copy not locked; this is probably a bug, please report")
 
+  /** @deprecated Unused, slated for removal in the next major release. */
   SVN_ERRDEF (SVN_ERR_WC_INVALID_LOCK,
               SVN_ERR_WC_CATEGORY_START + 6,
               "Invalid lock")
@@ -357,6 +374,21 @@ SVN_ERROR_START
               SVN_ERR_WC_CATEGORY_START + 22,
               "Path syntax not supported in this context")  
 
+  /** @since New in 1.2. */
+  SVN_ERRDEF (SVN_ERR_WC_INVALID_SCHEDULE,
+              SVN_ERR_WC_CATEGORY_START + 23,
+              "Invalid schedule")  
+
+  /** @since New in 1.3. */
+  SVN_ERRDEF (SVN_ERR_WC_INVALID_RELOCATION,
+              SVN_ERR_WC_CATEGORY_START + 24,
+              "Invalid relocation")
+
+  /** @since New in 1.3. */
+  SVN_ERRDEF (SVN_ERR_WC_INVALID_SWITCH,
+              SVN_ERR_WC_CATEGORY_START + 25,
+              "Invalid switch")
+
   /* fs errors */
 
   SVN_ERRDEF (SVN_ERR_FS_GENERAL,
@@ -491,6 +523,68 @@ SVN_ERROR_START
               SVN_ERR_FS_CATEGORY_START + 32,
               "Transaction is not dead")
 
+  /** @since New in 1.1. */
+  SVN_ERRDEF (SVN_ERR_FS_UNKNOWN_FS_TYPE,
+              SVN_ERR_FS_CATEGORY_START + 33,
+              "Unknown FS type")
+
+  /** @since New in 1.2. */
+  SVN_ERRDEF (SVN_ERR_FS_NO_USER,
+              SVN_ERR_FS_CATEGORY_START + 34,
+              "No user associated with filesystem")
+
+  /** @since New in 1.2. */
+  SVN_ERRDEF (SVN_ERR_FS_PATH_ALREADY_LOCKED,
+              SVN_ERR_FS_CATEGORY_START + 35,
+              "Path is already locked")
+
+  /** @since New in 1.2. */
+  SVN_ERRDEF (SVN_ERR_FS_PATH_NOT_LOCKED,
+              SVN_ERR_FS_CATEGORY_START + 36,
+              "Path is not locked")
+
+  /** @since New in 1.2. */
+  SVN_ERRDEF (SVN_ERR_FS_BAD_LOCK_TOKEN,
+              SVN_ERR_FS_CATEGORY_START + 37,
+              "Lock token is incorrect")
+
+  /** @since New in 1.2. */
+  SVN_ERRDEF (SVN_ERR_FS_NO_LOCK_TOKEN,
+              SVN_ERR_FS_CATEGORY_START + 38,
+              "No lock token provided")
+
+  /** @since New in 1.2. */
+  SVN_ERRDEF (SVN_ERR_FS_LOCK_OWNER_MISMATCH,
+              SVN_ERR_FS_CATEGORY_START + 39,
+              "Username does not match lock owner")
+
+  /** @since New in 1.2. */
+  SVN_ERRDEF (SVN_ERR_FS_NO_SUCH_LOCK,
+              SVN_ERR_FS_CATEGORY_START + 40,
+              "Filesystem has no such lock")
+
+  /** @since New in 1.2. */
+  SVN_ERRDEF (SVN_ERR_FS_LOCK_EXPIRED,
+              SVN_ERR_FS_CATEGORY_START + 41,
+              "Lock has expired")
+
+  /** @since New in 1.2. */
+  SVN_ERRDEF (SVN_ERR_FS_OUT_OF_DATE,
+              SVN_ERR_FS_CATEGORY_START + 42,
+              "Item is out of date")
+
+  /**@since New in 1.2.
+   *
+   * This is analogous to SVN_ERR_REPOS_UNSUPPORTED_VERSION.  To avoid
+   * confusion with "versions" (i.e., releases) of Subversion, we've
+   * started calling this the "format" number instead.  The old
+   * SVN_ERR_REPOS_UNSUPPORTED_VERSION error predates this and so
+   * retains its name.
+   */
+  SVN_ERRDEF (SVN_ERR_FS_UNSUPPORTED_FORMAT,
+              SVN_ERR_FS_CATEGORY_START + 43,
+              "Unsupported FS format")
+
   /* repos errors */
 
   SVN_ERRDEF (SVN_ERR_REPOS_LOCKED,
@@ -513,6 +607,11 @@ SVN_ERROR_START
               SVN_ERR_REPOS_CATEGORY_START + 4,
               "Bogus revision report")
  
+  /* This is analogous to SVN_ERR_FS_UNSUPPORTED_FORMAT.  To avoid
+   * confusion with "versions" (i.e., releases) of Subversion, we
+   * started using the word "format" instead of "version".  However,
+   * this error code's name predates that decision.
+   */
   SVN_ERRDEF (SVN_ERR_REPOS_UNSUPPORTED_VERSION,
               SVN_ERR_REPOS_CATEGORY_START + 5,
               "Unsupported repository version")
@@ -525,6 +624,17 @@ SVN_ERROR_START
               SVN_ERR_REPOS_CATEGORY_START + 7,
               "Error running post-commit hook")
 
+  /** @since New in 1.2. */
+  SVN_ERRDEF (SVN_ERR_REPOS_POST_LOCK_HOOK_FAILED,
+              SVN_ERR_REPOS_CATEGORY_START + 8,
+              "Error running post-lock hook")
+
+  /** @since New in 1.2. */
+  SVN_ERRDEF (SVN_ERR_REPOS_POST_UNLOCK_HOOK_FAILED,
+              SVN_ERR_REPOS_CATEGORY_START + 9,
+              "Error running post-unlock hook")
+
+
   /* generic RA errors */
 
   SVN_ERRDEF (SVN_ERR_RA_ILLEGAL_URL,
@@ -555,6 +665,12 @@ SVN_ERROR_START
               SVN_ERR_RA_CATEGORY_START + 6,
               "Unsupported RA plugin ABI version")
 
+  /** @since New in 1.2. */
+  SVN_ERRDEF (SVN_ERR_RA_NOT_LOCKED,
+              SVN_ERR_RA_CATEGORY_START + 7,
+              "Path is not locked")
+
+
   /* ra_dav errors */
 
   SVN_ERRDEF (SVN_ERR_RA_DAV_SOCK_INIT,
@@ -591,8 +707,17 @@ SVN_ERROR_START
 
   SVN_ERRDEF (SVN_ERR_RA_DAV_PROPPATCH_FAILED,
               SVN_ERR_RA_DAV_CATEGORY_START + 8,
-              "Failed to excute WebDAV PROPPATCH")
+              "Failed to execute WebDAV PROPPATCH")
 
+  /** @since New in 1.2. */
+  SVN_ERRDEF (SVN_ERR_RA_DAV_MALFORMED_DATA,
+              SVN_ERR_RA_DAV_CATEGORY_START + 9,
+              "Malformed network data")
+
+  /** @since New in 1.3 */
+  SVN_ERRDEF (SVN_ERR_RA_DAV_RESPONSE_HEADER_BADNESS,
+              SVN_ERR_RA_DAV_CATEGORY_START + 10,
+              "Unable to extract data from response header")
 
   /* ra_local errors */
   
@@ -659,6 +784,25 @@ SVN_ERROR_START
               SVN_ERR_AUTHZ_CATEGORY_START + 0,
               "Read access denied for root of edit")
 
+  /** @since New in 1.1. */
+  SVN_ERRDEF (SVN_ERR_AUTHZ_UNREADABLE,
+              SVN_ERR_AUTHZ_CATEGORY_START + 1,
+              "Item is not readable")
+
+  /** @since New in 1.1. */
+  SVN_ERRDEF (SVN_ERR_AUTHZ_PARTIALLY_READABLE,
+              SVN_ERR_AUTHZ_CATEGORY_START + 2,
+              "Item is partially readable")
+
+  SVN_ERRDEF (SVN_ERR_AUTHZ_INVALID_CONFIG,
+              SVN_ERR_AUTHZ_CATEGORY_START + 3,
+              "Invalid authz configuration")
+
+  /** @since New in 1.3 */
+  SVN_ERRDEF (SVN_ERR_AUTHZ_UNWRITABLE,
+              SVN_ERR_AUTHZ_CATEGORY_START + 4,
+              "Item is not writable")
+
   /* svndiff errors */
 
   SVN_ERRDEF (SVN_ERR_SVNDIFF_INVALID_HEADER,
@@ -721,11 +865,15 @@ SVN_ERROR_START
               SVN_ERR_CLIENT_CATEGORY_START + 3,
               "Attempting to commit to a URL more than once")
 
-  /* UNUSED error slot:                    + 4 */
+  SVN_ERRDEF (SVN_ERR_CLIENT_IS_BINARY_FILE,
+              SVN_ERR_CLIENT_CATEGORY_START + 4,
+              "Operation does not apply to binary file")
 
+       /*### SVN_PROP_EXTERNALS needed to be replaced with "svn:externals"
+         in order to get gettext translatable strings */
   SVN_ERRDEF (SVN_ERR_CLIENT_INVALID_EXTERNALS_DESCRIPTION,
               SVN_ERR_CLIENT_CATEGORY_START + 5,
-              "Format of an " SVN_PROP_EXTERNALS " property was invalid")
+              "Format of an svn:externals property was invalid")
 
   SVN_ERRDEF (SVN_ERR_CLIENT_MODIFIED,
               SVN_ERR_CLIENT_CATEGORY_START + 6,
@@ -745,12 +893,22 @@ SVN_ERROR_START
 
   SVN_ERRDEF (SVN_ERR_CLIENT_REVISION_AUTHOR_CONTAINS_NEWLINE,
               SVN_ERR_CLIENT_CATEGORY_START + 10,
-              SVN_PROP_REVISION_AUTHOR " contains a newline")
-                   
+              "Author name cannot contain a newline")
+
   SVN_ERRDEF (SVN_ERR_CLIENT_PROPERTY_NAME,
               SVN_ERR_CLIENT_CATEGORY_START + 11,
               "Bad property name")
 
+  /** @since New in 1.1. */
+  SVN_ERRDEF (SVN_ERR_CLIENT_UNRELATED_RESOURCES,
+              SVN_ERR_CLIENT_CATEGORY_START + 12,
+              "Two versioned resources are unrelated")
+
+  /** @since New in 1.2. */
+  SVN_ERRDEF (SVN_ERR_CLIENT_MISSING_LOCK_TOKEN,
+              SVN_ERR_CLIENT_CATEGORY_START + 13,
+              "Path has no lock token")
+
   /* misc errors */
 
   SVN_ERRDEF (SVN_ERR_BASE,
@@ -829,6 +987,11 @@ SVN_ERROR_START
               SVN_ERR_MISC_CATEGORY_START + 18,
               "No auth file path available")
 
+  /** @since New in 1.1. */
+  SVN_ERRDEF (SVN_ERR_VERSION_MISMATCH,
+              SVN_ERR_MISC_CATEGORY_START + 19,
+              "Incompatible library version")
+
   /* command-line client errors */
 
   SVN_ERRDEF (SVN_ERR_CL_ARG_PARSING_ERROR,
@@ -867,6 +1030,10 @@ SVN_ERROR_START
               SVN_ERR_CL_CATEGORY_START + 8,
               "Something is wrong with the log message's contents")
 
+  SVN_ERRDEF (SVN_ERR_CL_UNNECESSARY_LOG_MESSAGE,
+              SVN_ERR_CL_CATEGORY_START + 9,
+              "A log message was given where none was necessary")
+
 SVN_ERROR_END
 
 
