@@ -92,8 +92,9 @@ get_wc_prop (void *baton,
       int i;
       for (i = 0; i < cb->commit_items->nelts; i++)
         {
-          svn_client_commit_item_t *item
-            = ((svn_client_commit_item_t **) cb->commit_items->elts)[i];
+          svn_client_commit_item2_t *item
+            = APR_ARRAY_IDX(cb->commit_items, i,
+                            svn_client_commit_item2_t *);
           if (! strcmp (relpath, 
                         svn_path_uri_decode (item->url, pool)))
             return svn_wc_prop_get (value, name, item->path, cb->base_access,
@@ -133,8 +134,8 @@ push_wc_prop (void *baton,
 
   for (i = 0; i < cb->commit_items->nelts; i++)
     {
-      svn_client_commit_item_t *item
-        = ((svn_client_commit_item_t **) cb->commit_items->elts)[i];
+      svn_client_commit_item2_t *item
+        = APR_ARRAY_IDX(cb->commit_items, i, svn_client_commit_item2_t *);
       
       if (strcmp (relpath, svn_path_uri_decode (item->url, pool)) == 0)
         {
@@ -259,17 +260,17 @@ invalidate_wc_props (void *baton,
 
 
 svn_error_t * 
-svn_client__open_ra_session (svn_ra_session_t **ra_session,
-                             const char *base_url,
-                             const char *base_dir,
-                             svn_wc_adm_access_t *base_access,
-                             apr_array_header_t *commit_items,
-                             svn_boolean_t use_admin,
-                             svn_boolean_t read_only_wc,
-                             svn_client_ctx_t *ctx,
-                             apr_pool_t *pool)
+svn_client__open_ra_session_internal (svn_ra_session_t **ra_session,
+                                      const char *base_url,
+                                      const char *base_dir,
+                                      svn_wc_adm_access_t *base_access,
+                                      apr_array_header_t *commit_items,
+                                      svn_boolean_t use_admin,
+                                      svn_boolean_t read_only_wc,
+                                      svn_client_ctx_t *ctx,
+                                      apr_pool_t *pool)
 {
-  svn_ra_callbacks_t *cbtable = apr_pcalloc (pool, sizeof(*cbtable));
+  svn_ra_callbacks2_t *cbtable = apr_pcalloc (pool, sizeof(*cbtable));
   svn_client__callback_baton_t *cb = apr_pcalloc (pool, sizeof(*cb));
   
   cbtable->open_tmp_file = use_admin ? open_admin_tmp_file : open_tmp_file;
@@ -278,6 +279,8 @@ svn_client__open_ra_session (svn_ra_session_t **ra_session,
   cbtable->push_wc_prop = commit_items ? push_wc_prop : NULL;
   cbtable->invalidate_wc_props = read_only_wc ? NULL : invalidate_wc_props;
   cbtable->auth_baton = ctx->auth_baton; /* new-style */
+  cbtable->progress_func = ctx->progress_func;
+  cbtable->progress_baton = ctx->progress_baton;
 
   cb->base_dir = base_dir;
   cb->base_access = base_access;
@@ -285,11 +288,21 @@ svn_client__open_ra_session (svn_ra_session_t **ra_session,
   cb->commit_items = commit_items;
   cb->ctx = ctx;
 
-  SVN_ERR (svn_ra_open (ra_session, base_url, cbtable, cb, ctx->config, pool));
+  SVN_ERR (svn_ra_open2 (ra_session, base_url, cbtable, cb,
+                         ctx->config, pool));
 
   return SVN_NO_ERROR;
 }
 
+svn_error_t *
+svn_client_open_ra_session (svn_ra_session_t **session,
+                            const char *url,
+                            svn_client_ctx_t *ctx,
+                            apr_pool_t *pool)
+{
+  return svn_client__open_ra_session_internal (session, url, NULL, NULL, NULL,
+                                               FALSE, TRUE, ctx, pool);
+}
 
 
 svn_error_t *
@@ -302,10 +315,10 @@ svn_client_uuid_from_url (const char **uuid,
   apr_pool_t *subpool = svn_pool_create (pool);
 
   /* use subpool to create a temporary RA session */
-  SVN_ERR (svn_client__open_ra_session (&ra_session, url,
-                                        NULL, /* no base dir */
-                                        NULL, NULL, FALSE, TRUE, 
-                                        ctx, subpool));
+  SVN_ERR (svn_client__open_ra_session_internal (&ra_session, url,
+                                                 NULL, /* no base dir */
+                                                 NULL, NULL, FALSE, TRUE, 
+                                                 ctx, subpool));
 
   SVN_ERR (svn_ra_get_uuid (ra_session, uuid, subpool));
 
@@ -340,11 +353,28 @@ svn_client_uuid_from_path (const char **uuid,
     {
       *uuid = entry->uuid;
     }
-  else
+  else if (entry->url)
     {
       /* fallback to using the network. */
       SVN_ERR (svn_client_uuid_from_url (uuid, entry->url, ctx, pool));
     }
+  else
+    {
+      /* Try the parent if it's the same working copy.  It's not
+         entirely clear how this happens (possibly an old wc?) but it
+         has been triggered by TSVN, see
+         http://subversion.tigris.org/servlets/ReadMsg?list=dev&msgNo=101831
+         Message-ID: <877jgjtkus.fsf@debian2.lan> */
+      svn_boolean_t is_root;
+      SVN_ERR (svn_wc_is_wc_root (&is_root, path, adm_access, pool));
+      if (is_root)
+        return svn_error_createf (SVN_ERR_ENTRY_MISSING_URL, NULL,
+                                  _("'%s' has no URL"),
+                                  svn_path_local_style (path, pool));
+      else
+        return svn_client_uuid_from_path (uuid, svn_path_dirname (path, pool),
+                                          adm_access, ctx, pool);
+    }
 
   return SVN_NO_ERROR;
 }
@@ -714,9 +744,9 @@ svn_client__repos_locations (const char **start_url,
      don't need to do anything more here in that case. */
 
   /* Open a RA session to this URL. */
-  SVN_ERR (svn_client__open_ra_session (&ra_session, url, NULL,
-                                        NULL, NULL, FALSE, TRUE,
-                                        ctx, subpool));
+  SVN_ERR (svn_client__open_ra_session_internal (&ra_session, url, NULL,
+                                                 NULL, NULL, FALSE, TRUE,
+                                                 ctx, subpool));
 
   /* Resolve the opt_revision_ts. */
   if (peg_revnum == SVN_INVALID_REVNUM)
@@ -783,12 +813,12 @@ svn_client__repos_locations (const char **start_url,
   /* We'd better have all the paths we were looking for! */
   if (! start_path)
     return svn_error_createf 
-      (APR_EGENERAL, NULL,
+      (SVN_ERR_CLIENT_UNRELATED_RESOURCES, NULL,
        _("Unable to find repository location for '%s' in revision %ld"),
        path, start_revnum);
   if (! end_path)
     return svn_error_createf 
-      (APR_EGENERAL, NULL,
+      (SVN_ERR_CLIENT_UNRELATED_RESOURCES, NULL,
        _("The location for '%s' for revision %ld does not exist in the "
          "repository or refers to an unrelated object"),
        path, end_revnum);
@@ -886,9 +916,9 @@ svn_client__ra_session_from_path (svn_ra_session_t **ra_session_p,
                                         ctx, pool));
   good_rev = new_rev;
 
-  SVN_ERR (svn_client__open_ra_session (&ra_session, url,
-                                        NULL, NULL, NULL, FALSE, FALSE,
-                                        ctx, pool));
+  SVN_ERR (svn_client__open_ra_session_internal (&ra_session, url,
+                                                 NULL, NULL, NULL,
+                                                 FALSE, FALSE, ctx, pool));
 
   /* Resolve good_rev into a real revnum. */
   SVN_ERR (svn_client__get_revision_number (&rev, ra_session,
