@@ -219,7 +219,7 @@ svn_io_open_unique_file (apr_file_t **f,
       SVN_ERR (svn_path_cstring_from_utf8 (&unique_name_apr, unique_name,
                                            pool));
 
-      apr_err = apr_file_open (f, unique_name_apr, flag,
+      apr_err = apr_file_open (f, unique_name_apr, flag | APR_BINARY,
                                APR_OS_DEFAULT, pool);
 
       if (APR_STATUS_IS_EEXIST (apr_err))
@@ -1000,9 +1000,12 @@ reown_file (const char *path_apr,
   return SVN_NO_ERROR;
 }
 
-/* Set PERMS to the default file permissions for PATH based on the
-   permissions of PATH and a newly created file.  Make temporary
-   allocations in POOL.  */
+/* Determine what the read-write PERMS for PATH should be by ORing
+   together the permissions of PATH and the permissions of a temporary
+   file that we create.  Unfortunately, this is the only way to
+   determine which combination of write bits (User/Group/World) should
+   be set to restore a file from read-only to read-write.  Make
+   temporary allocations in POOL.  */
 static svn_error_t *
 get_default_file_perms (const char *path, apr_fileperms_t *perms,
                         apr_pool_t *pool)
@@ -1016,7 +1019,7 @@ get_default_file_perms (const char *path, apr_fileperms_t *perms,
   /* Get the perms for a newly created file to find out what write
    * bits should be set. */
   SVN_ERR (svn_io_open_unique_file (&fd, &tmp_path, path, 
-                                    "tmp", TRUE, pool));
+                                    ".tmp", TRUE, pool));
   status = apr_stat (&tmp_finfo, tmp_path, APR_FINFO_PROT, pool);
   if (status)
     return svn_error_wrap_apr (status, _("Can't get default file perms "
@@ -1027,9 +1030,10 @@ get_default_file_perms (const char *path, apr_fileperms_t *perms,
   /* Get the perms for the original file so we'll have any other bits
    * that were already set (like the execute bits, for example). */
   SVN_ERR (svn_path_cstring_from_utf8 (&apr_path, path, pool));
-  status = apr_file_open (&fd, apr_path, APR_READ, APR_OS_DEFAULT, pool);
+  status = apr_file_open (&fd, apr_path, APR_READ | APR_BINARY,
+                          APR_OS_DEFAULT, pool);
   if (status)
-    return svn_error_wrap_apr (status, _("Can't open file at '%s' "), path);
+    return svn_error_wrap_apr (status, _("Can't open file at '%s'"), path);
 
   status = apr_stat (&finfo, apr_path, APR_FINFO_PROT, pool);
   if (status)
@@ -1650,19 +1654,15 @@ svn_io_remove_dir (const char *path, apr_pool_t *pool)
 
 
 svn_error_t *
-svn_io_get_dirents (apr_hash_t **dirents,
-                    const char *path,
-                    apr_pool_t *pool)
+svn_io_get_dirents2 (apr_hash_t **dirents,
+                     const char *path,
+                     apr_pool_t *pool)
 {
   apr_status_t status; 
   apr_dir_t *this_dir;
   apr_finfo_t this_entry;
   apr_int32_t flags = APR_FINFO_TYPE | APR_FINFO_NAME;
 
-  /* These exist so we can use their addresses as hash values! */
-  static const svn_node_kind_t static_svn_node_file = svn_node_file;
-  static const svn_node_kind_t static_svn_node_dir = svn_node_dir;
-
   *dirents = apr_hash_make (pool);
   
   SVN_ERR (svn_io_dir_open (&this_dir, path, pool));
@@ -1681,20 +1681,26 @@ svn_io_get_dirents (apr_hash_t **dirents,
       else
         {
           const char *name;
+          svn_io_dirent_t *dirent = apr_pcalloc (pool, sizeof (*dirent));
 
           SVN_ERR (svn_path_cstring_to_utf8 (&name, this_entry.name, pool));
           
           if (this_entry.filetype == APR_REG)
-            apr_hash_set (*dirents, name, APR_HASH_KEY_STRING,
-                          &static_svn_node_file);
+            dirent->kind = svn_node_file;
           else if (this_entry.filetype == APR_DIR)
-            apr_hash_set (*dirents, name, APR_HASH_KEY_STRING,
-                          &static_svn_node_dir);
+            dirent->kind = svn_node_dir;
+          else if (this_entry.filetype == APR_LNK)
+            {
+              dirent->kind = svn_node_file;
+              dirent->special = TRUE;
+            }
           else
-            /* ### symlinks, etc. will fall into this category for now.
-               someday subversion will recognize them. :)  */
-            apr_hash_set (*dirents, name, APR_HASH_KEY_STRING,
-                          &static_svn_node_file);
+            /* ### Currently, Subversion supports just symlinks; other
+             * entry types are reported as regular files. This is inconsistent
+             * with svn_io_check_path(). */
+            dirent->kind = svn_node_file;
+
+          apr_hash_set (*dirents, name, APR_HASH_KEY_STRING, dirent);
         }
     }
 
@@ -1710,24 +1716,32 @@ svn_io_get_dirents (apr_hash_t **dirents,
   return SVN_NO_ERROR;
 }
 
+svn_error_t *
+svn_io_get_dirents (apr_hash_t **dirents,
+                    const char *path,
+                    apr_pool_t *pool)
+{
+  /* Note that in C, padding is not allowed at the beginning of structs,
+     so this is actually portable, since the kind field of svn_io_dirent_t
+     is first in that struct. */
+  return svn_io_get_dirents2 (dirents, path, pool);
+}
+
 
 svn_error_t *
-svn_io_run_cmd (const char *path,
-                const char *cmd,
-                const char *const *args,
-                int *exitcode,
-                apr_exit_why_e *exitwhy,
-                svn_boolean_t inherit,
-                apr_file_t *infile,
-                apr_file_t *outfile,
-                apr_file_t *errfile,
-                apr_pool_t *pool)
+svn_io_start_cmd (apr_proc_t *cmd_proc,
+                  const char *path,
+                  const char *cmd,
+                  const char *const *args,
+                  svn_boolean_t inherit,
+                  apr_file_t *infile,
+                  apr_file_t *outfile,
+                  apr_file_t *errfile,
+                  apr_pool_t *pool)
 {
   apr_status_t apr_err;
-  apr_proc_t cmd_proc;
   apr_procattr_t *cmdproc_attr;
-  apr_exit_why_e exitwhy_val;
-  int exitcode_val, num_args;
+  int num_args;
   const char **args_native;
   const char *cmd_apr;
 
@@ -1801,16 +1815,30 @@ svn_io_run_cmd (const char *path,
 
 
   /* Start the cmd command. */ 
-  apr_err = apr_proc_create (&cmd_proc, cmd_apr, args_native, NULL,
+  apr_err = apr_proc_create (cmd_proc, cmd_apr, args_native, NULL,
                              cmdproc_attr, pool);
   if (apr_err)
     return svn_error_wrap_apr (apr_err, _("Can't start process '%s'"), cmd);
 
+  return SVN_NO_ERROR;
+}
+
+svn_error_t *
+svn_io_wait_for_cmd (apr_proc_t *cmd_proc,
+                     const char *cmd,
+                     int *exitcode,
+                     apr_exit_why_e *exitwhy,
+                     apr_pool_t *pool)
+{
+  apr_status_t apr_err;
+  apr_exit_why_e exitwhy_val;
+  int exitcode_val;
+
   /* The Win32 apr_proc_wait doesn't set this... */
   exitwhy_val = APR_PROC_EXIT;
 
   /* Wait for the cmd command to finish. */
-  apr_err = apr_proc_wait (&cmd_proc, &exitcode_val, &exitwhy_val, APR_WAIT);
+  apr_err = apr_proc_wait (cmd_proc, &exitcode_val, &exitwhy_val, APR_WAIT);
   if (APR_STATUS_IS_CHILD_NOTDONE (apr_err))
     return svn_error_wrap_apr (apr_err, _("Error waiting for process '%s'"),
                                cmd);
@@ -1833,6 +1861,29 @@ svn_io_run_cmd (const char *path,
 }
 
 
+svn_error_t *
+svn_io_run_cmd (const char *path,
+                const char *cmd,
+                const char *const *args,
+                int *exitcode,
+                apr_exit_why_e *exitwhy,
+                svn_boolean_t inherit,
+                apr_file_t *infile,
+                apr_file_t *outfile,
+                apr_file_t *errfile,
+                apr_pool_t *pool)
+{
+  apr_proc_t cmd_proc;
+
+  SVN_ERR (svn_io_start_cmd (&cmd_proc, path, cmd, args, inherit,
+                             infile, outfile, errfile, pool));
+
+  SVN_ERR (svn_io_wait_for_cmd (&cmd_proc, cmd, exitcode, exitwhy, pool));
+
+  return SVN_NO_ERROR;
+}
+
+
 svn_error_t *
 svn_io_run_diff (const char *dir, 
                  const char *const *user_args,
@@ -2118,7 +2169,7 @@ svn_io_file_open (apr_file_t **new_file, const char *fname,
   apr_status_t status;
 
   SVN_ERR (svn_path_cstring_from_utf8 (&fname_apr, fname, pool));
-  status = apr_file_open (new_file, fname_apr, flag, perm, pool);
+  status = apr_file_open (new_file, fname_apr, flag | APR_BINARY, perm, pool);
 
   if (status)
     return svn_error_wrap_apr (status, _("Can't open file '%s'"),
@@ -2145,10 +2196,10 @@ do_io_file_wrapper_cleanup (apr_file_t *file, apr_status_t status,
   svn_error_clear (err);
 
   if (name)
-    return svn_error_wrap_apr (status, gettext(msg),
+    return svn_error_wrap_apr (status, _(msg),
                                svn_path_local_style (name, pool));
   else
-    return svn_error_wrap_apr (status, gettext(msg_no_name));
+    return svn_error_wrap_apr (status, _(msg_no_name));
 }
 
 
@@ -2375,6 +2426,61 @@ svn_io_file_rename (const char *from_path, const char *to_path,
 }
 
 
+svn_error_t *
+svn_io_file_move (const char *from_path, const char *to_path,
+                  apr_pool_t *pool)
+{
+  svn_error_t *err = svn_io_file_rename (from_path, to_path, pool);
+
+  if (err && APR_STATUS_IS_EXDEV (err->apr_err))
+    {
+      const char *tmp_to_path;
+      apr_file_t *fp;
+
+      svn_error_clear (err);
+
+      SVN_ERR (svn_io_open_unique_file (&fp, &tmp_to_path,
+                                        to_path, "tmp", FALSE, pool));
+      apr_file_close (fp);
+
+      err = svn_io_copy_file (from_path, tmp_to_path, TRUE, pool);
+      if (err)
+        goto failed_tmp;
+
+      err = svn_io_file_rename (tmp_to_path, to_path, pool);
+      if (err)
+        goto failed_tmp;
+
+#ifdef WIN32
+      /* Windows won't let us delete a file marked as read-only,
+         so, mark as read+write */
+      err = svn_io_set_file_read_write (from_path, FALSE, pool);
+      if (err)
+        goto failed_final;
+#endif
+
+      err = svn_io_remove_file (from_path, pool);
+      if (! err)
+        return SVN_NO_ERROR;
+
+#ifdef WIN32
+    failed_final:
+      svn_error_clear (svn_io_set_file_read_write (to_path, FALSE, pool));
+#endif
+      svn_error_clear (svn_io_remove_file (to_path, pool));
+
+      return err;
+
+    failed_tmp:
+#ifdef WIN32
+      svn_error_clear (svn_io_set_file_read_write (tmp_to_path, FALSE, pool));
+#endif
+      svn_error_clear (svn_io_remove_file (tmp_to_path, pool));
+    }
+
+  return err;
+}
+
 /* Common implementation of svn_io_dir_make and svn_io_dir_make_hidden.
    HIDDEN determines if the hidden attribute
    should be set on the newly created directory. */
@@ -2657,9 +2763,12 @@ svn_io_dir_walk (const char *dirname,
  * @param pool Used for temporary allocation.
  * @remark If path is not a directory, or some other error occurs,
  * then return the appropriate apr status code.
+ *
+ * (This function is written in APR style, in anticipation of
+ * perhaps someday being moved to APR as 'apr_dir_is_empty'.)
  */                        
 static apr_status_t
-apr_dir_is_empty (const char *dir, apr_pool_t *pool)
+dir_is_empty (const char *dir, apr_pool_t *pool)
 {
   apr_status_t apr_err;
   apr_dir_t *dir_handle;
@@ -2673,9 +2782,6 @@ apr_dir_is_empty (const char *dir, apr_pool_t *pool)
   apr_err = apr_dir_open (&dir_handle, dir, pool);
   if (apr_err != APR_SUCCESS)
     return apr_err;
-      
-  /* ### What is the gospel on the APR_STATUS_IS_SUCCESS macro these
-     days? :-) */
 
   for (apr_err = apr_dir_read (&finfo, APR_FINFO_NAME, dir_handle);
        apr_err == APR_SUCCESS;
@@ -2716,7 +2822,7 @@ svn_io_dir_empty (svn_boolean_t *is_empty_p,
 
   SVN_ERR (svn_path_cstring_from_utf8 (&path_apr, path, pool));
 
-  status = apr_dir_is_empty (path_apr, pool);
+  status = dir_is_empty (path_apr, pool);
 
   if (!status)
     *is_empty_p = TRUE;
@@ -2854,8 +2960,11 @@ contents_identical_p (svn_boolean_t *identical_p,
       err2 = svn_io_file_read_full (file2_h, buf2, 
                                     sizeof(buf2), &bytes_read2, pool);
       if (err2 && !APR_STATUS_IS_EOF(err2->apr_err))
-        return err2;
-      
+        {
+          svn_error_clear (err1);
+          return err2;
+        }
+
       if ((bytes_read1 != bytes_read2)
           || (memcmp (buf1, buf2, bytes_read1)))
         {
