@@ -2,7 +2,7 @@
  * status.c: construct a status structure from an entry structure
  *
  * ====================================================================
- * Copyright (c) 2000-2004 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2006 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -33,9 +33,11 @@
 #include "svn_io.h"
 #include "svn_wc.h"
 #include "svn_config.h"
+#include "svn_time.h"
 #include "svn_private_config.h"
 
 #include "wc.h"
+#include "lock.h"
 #include "props.h"
 #include "translate.h"
 
@@ -131,6 +133,15 @@ struct dir_baton
 
   /* The pool in which this baton itself is allocated. */
   apr_pool_t *pool;
+
+  /* The URI to this item in the repository. */
+  const char *url;
+
+  /* Out of date info corresponding to ood_* fields in svn_wc_status2_t. */
+  svn_revnum_t ood_last_cmt_rev;
+  apr_time_t ood_last_cmt_date;
+  svn_node_kind_t ood_kind;
+  const char *ood_last_cmt_author;
 };
 
 
@@ -165,6 +176,14 @@ struct file_baton
      the code that syncs up the adm dir and working copy. */
   svn_boolean_t prop_changed;
 
+  /* The URI to this item in the repository. */
+  const char *url;
+
+  /* Out of date info corresponding to ood_* fields in svn_wc_status2_t. */
+  svn_revnum_t ood_last_cmt_rev;
+  apr_time_t ood_last_cmt_date;
+  svn_node_kind_t ood_kind;
+  const char *ood_last_cmt_author;
 };
 
 
@@ -188,6 +207,9 @@ struct file_baton
    NOTE: this may be svn_node_unknown if the caller has made no such
    determination.
 
+   If PATH_KIND is not svn_node_unknown, PATH_SPECIAL indicates whether
+   the entry is a special file.
+
    If GET_ALL is zero, and ENTRY is not locally modified, then *STATUS
    will be set to NULL.  If GET_ALL is non-zero, then *STATUS will be
    allocated and returned no matter what.
@@ -206,7 +228,7 @@ assemble_status (svn_wc_status2_t **status,
                  svn_wc_adm_access_t *adm_access,
                  const svn_wc_entry_t *entry,
                  const svn_wc_entry_t *parent_entry,
-                 svn_node_kind_t path_kind,
+                 svn_node_kind_t path_kind, svn_boolean_t path_special,
                  svn_boolean_t get_all,
                  svn_boolean_t is_ignored,
                  apr_hash_t *repos_locks,
@@ -219,9 +241,9 @@ assemble_status (svn_wc_status2_t **status,
   svn_boolean_t prop_modified_p = FALSE;
   svn_boolean_t locked_p = FALSE;
   svn_boolean_t switched_p = FALSE;
+#ifdef HAVE_SYMLINK          
   svn_boolean_t wc_special;
-  svn_boolean_t node_special;
-  svn_node_kind_t kind;
+#endif /* HAVE_SYMLINK */
 
   /* Defaults for two main variables. */
   enum svn_wc_status_kind final_text_status = svn_wc_status_normal;
@@ -250,8 +272,8 @@ assemble_status (svn_wc_status2_t **status,
 
   /* Check the path kind for PATH. */
   if (path_kind == svn_node_unknown)
-    SVN_ERR (svn_io_check_path (path, &path_kind, pool));
-  SVN_ERR (svn_io_check_special_path (path, &kind, &node_special, pool));
+    SVN_ERR (svn_io_check_special_path (path, &path_kind, &path_special,
+                                        pool));
   
   if (! entry)
     {
@@ -280,6 +302,11 @@ assemble_status (svn_wc_status2_t **status,
         }
 
       stat->repos_lock = repos_lock;
+      stat->url = NULL;
+      stat->ood_last_cmt_rev = SVN_INVALID_REVNUM;
+      stat->ood_last_cmt_date = 0;
+      stat->ood_kind = svn_node_none;
+      stat->ood_last_cmt_author = NULL;
 
       *status = stat;
       return SVN_NO_ERROR;
@@ -335,15 +362,17 @@ assemble_status (svn_wc_status2_t **status,
       SVN_ERR (svn_wc_props_modified_p (&prop_modified_p, path, adm_access,
                                         pool));
 
+#ifdef HAVE_SYMLINK
       if (has_props)
         SVN_ERR (svn_wc__get_special (&wc_special, path, adm_access, pool));
       else
         wc_special = FALSE;
+#endif /* HAVE_SYMLINK */
 
       /* If the entry is a file, check for textual modifications */
       if ((entry->kind == svn_node_file)
 #ifdef HAVE_SYMLINK          
-          && (wc_special == node_special)
+          && (wc_special == path_special)
 #endif /* HAVE_SYMLINK */          
           )
         SVN_ERR (svn_wc_text_modified_p (&text_modified_p, path, FALSE,
@@ -425,12 +454,12 @@ assemble_status (svn_wc_status2_t **status,
         }
       else if (path_kind != entry->kind)
         final_text_status = svn_wc_status_obstructed;
-      else if (((! wc_special) && (node_special))
 #ifdef HAVE_SYMLINK      
-               || (wc_special && (! node_special))
-#endif /* HAVE_SYMLINK */
+      else if (((! wc_special) && (path_special))
+               || (wc_special && (! path_special))
                )
         final_text_status = svn_wc_status_obstructed;
+#endif /* HAVE_SYMLINK */
 
       if (path_kind == svn_node_dir && entry->kind == svn_node_dir)
         SVN_ERR (svn_wc_locked (&locked_p, path, pool));
@@ -464,6 +493,11 @@ assemble_status (svn_wc_status2_t **status,
   stat->switched = switched_p;
   stat->copied = entry->copied;
   stat->repos_lock = repos_lock;
+  stat->url = (entry->url ? entry->url : NULL);
+  stat->ood_last_cmt_rev = SVN_INVALID_REVNUM;
+  stat->ood_last_cmt_date = 0;
+  stat->ood_kind = svn_node_none;
+  stat->ood_last_cmt_author = NULL;
 
   *status = stat;
 
@@ -482,6 +516,7 @@ send_status_structure (const char *path,
                        const svn_wc_entry_t *entry,
                        const svn_wc_entry_t *parent_entry,
                        svn_node_kind_t path_kind,
+                       svn_boolean_t path_special,
                        svn_boolean_t get_all,
                        svn_boolean_t is_ignored,
                        apr_hash_t *repos_locks,
@@ -493,8 +528,8 @@ send_status_structure (const char *path,
   svn_wc_status2_t *statstruct;
   
   SVN_ERR (assemble_status (&statstruct, path, adm_access, entry, parent_entry,
-                            path_kind, get_all, is_ignored, repos_locks,
-                            repos_root, pool));
+                            path_kind, path_special, get_all, is_ignored,
+                            repos_locks, repos_root, pool));
   if (statstruct && (status_func))
     (*status_func) (status_baton, path, statstruct);
   
@@ -520,7 +555,7 @@ send_status_structure (const char *path,
    None of the arguments may be NULL.
 */
 static svn_error_t *
-collect_ignore_patterns (apr_array_header_t *patterns,
+collect_ignore_patterns (apr_array_header_t **patterns,
                          apr_array_header_t *ignores,
                          svn_wc_adm_access_t *adm_access,
                          apr_pool_t *pool)
@@ -528,11 +563,13 @@ collect_ignore_patterns (apr_array_header_t *patterns,
   int i;
   const svn_string_t *value;
 
+  *patterns = apr_array_make (pool, 1, sizeof (const char *));
+
   /* Copy default ignores into the local PATTERNS array. */
   for (i = 0; i < ignores->nelts; i++)
     {
       const char *ignore = APR_ARRAY_IDX (ignores, i, const char *);
-      APR_ARRAY_PUSH (patterns, const char *) = ignore;
+      APR_ARRAY_PUSH (*patterns, const char *) = ignore;
     }
 
   /* Then add any svn:ignore globs to the PATTERNS array. */
@@ -540,7 +577,7 @@ collect_ignore_patterns (apr_array_header_t *patterns,
                             svn_wc_adm_access_path (adm_access), adm_access,
                             pool));
   if (value != NULL)
-    svn_cstring_split_append (patterns, value->data, "\n\r", FALSE, pool);
+    svn_cstring_split_append (*patterns, value->data, "\n\r", FALSE, pool);
 
   return SVN_NO_ERROR;   
 } 
@@ -581,7 +618,8 @@ is_external_path (apr_hash_t *externals,
 
    NAME is the basename of the unversioned file whose status is being
    requested.  PATH_KIND is the node kind of NAME as determined by the
-   caller.  ADM_ACCESS is an access baton for the working copy path.
+   caller.  PATH_SPECIAL is the special status of the path, also determined
+   by the caller.  ADM_ACCESS is an access baton for the working copy path.
    PATTERNS points to a list of filename patterns which are marked as
    ignored.  None of these parameter may be NULL.  EXTERNALS is a hash
    of known externals definitions for this status run.
@@ -594,7 +632,7 @@ is_external_path (apr_hash_t *externals,
 */
 static svn_error_t *
 send_unversioned_item (const char *name, 
-                       svn_node_kind_t path_kind, 
+                       svn_node_kind_t path_kind, svn_boolean_t path_special,
                        svn_wc_adm_access_t *adm_access, 
                        apr_array_header_t *patterns,
                        apr_hash_t *externals,
@@ -612,8 +650,8 @@ send_unversioned_item (const char *name,
   svn_wc_status2_t *status;
 
   SVN_ERR (assemble_status (&status, path, adm_access, NULL, NULL, 
-                            path_kind, FALSE, ignore_me, repos_locks,
-                            repos_root, pool));
+                            path_kind, path_special, FALSE, ignore_me,
+                            repos_locks, repos_root, pool));
 
   if (is_external)
     status->text_status = svn_wc_status_external;
@@ -653,6 +691,8 @@ handle_dir_entry (struct edit_baton *eb,
                   const char *name,
                   const svn_wc_entry_t *dir_entry,
                   const svn_wc_entry_t *entry,
+                  svn_node_kind_t kind,
+                  svn_boolean_t special,
                   apr_array_header_t *ignores,
                   svn_boolean_t descend,
                   svn_boolean_t get_all,
@@ -665,10 +705,7 @@ handle_dir_entry (struct edit_baton *eb,
 {
   const char *dirname = svn_wc_adm_access_path (adm_access);
   const char *path = svn_path_join (dirname, name, pool);
-  svn_node_kind_t kind;
 
-  /* Get the entry's kind on disk. */
-  SVN_ERR (svn_io_check_path (path, &kind, pool));
   if (kind == svn_node_dir)
     {
       /* Directory entries are incomplete.  We must get their full
@@ -698,8 +735,9 @@ handle_dir_entry (struct edit_baton *eb,
       else
         {
           SVN_ERR (send_status_structure (path, adm_access, full_entry, 
-                                          dir_entry, kind, get_all, FALSE,
-                                          eb->repos_locks, eb->repos_root,
+                                          dir_entry, kind, special, get_all,
+                                          FALSE, eb->repos_locks,
+                                          eb->repos_root,
                                           status_func, status_baton, pool));
         }
     }
@@ -707,9 +745,9 @@ handle_dir_entry (struct edit_baton *eb,
     {
       /* File entries are ... just fine! */
       SVN_ERR (send_status_structure (path, adm_access, entry, dir_entry, 
-                                      kind, get_all, FALSE, eb->repos_locks,
-                                      eb->repos_root, status_func,
-                                      status_baton, pool));
+                                      kind, special, get_all, FALSE,
+                                      eb->repos_locks, eb->repos_root,
+                                      status_func, status_baton, pool));
     }
   return SVN_NO_ERROR;
 }
@@ -747,7 +785,7 @@ get_dir_status (struct edit_baton *eb,
   apr_hash_t *entries;
   apr_hash_index_t *hi;
   const svn_wc_entry_t *dir_entry;
-  const char *fullpath, *path = svn_wc_adm_access_path (adm_access);
+  const char *path = svn_wc_adm_access_path (adm_access);
   apr_hash_t *dirents;
   apr_array_header_t *patterns = NULL;
   apr_pool_t *iterpool, *subpool = svn_pool_create (pool);
@@ -768,11 +806,8 @@ get_dir_status (struct edit_baton *eb,
   /* Unless specified, add default ignore regular expressions and try
      to add any svn:ignore properties from the parent directory. */
   if (ignores)
-    {
-      patterns = apr_array_make (subpool, 1, sizeof (const char *));
-      SVN_ERR (collect_ignore_patterns (patterns, ignores, 
-                                        adm_access, subpool));
-    }
+    SVN_ERR (collect_ignore_patterns (&patterns, ignores, 
+                                      adm_access, subpool));
 
   /* If "this dir" has "svn:externals" property set on it, store its
      name and value in traversal_info.  Also, we want to track the
@@ -820,23 +855,26 @@ get_dir_status (struct edit_baton *eb,
   if (entry)
     {
       const svn_wc_entry_t *entry_entry;
+      svn_io_dirent_t* dirent_p = apr_hash_get (dirents, entry,
+                                                APR_HASH_KEY_STRING);
       entry_entry = apr_hash_get (entries, entry, APR_HASH_KEY_STRING);
 
       /* If ENTRY is versioned, send its versioned status. */
       if (entry_entry)
         {
           SVN_ERR (handle_dir_entry (eb, adm_access, entry, dir_entry, 
-                                     entry_entry, ignores, descend, get_all, 
+                                     entry_entry,
+                                     dirent_p ? dirent_p->kind : svn_node_none,
+                                     dirent_p ? dirent_p->special : FALSE,
+                                     ignores, descend, get_all, 
                                      no_ignore, status_func, status_baton, 
                                      cancel_func, cancel_baton, subpool));
         }
       /* Otherwise, if it exists, send its unversioned status. */
-      else if (apr_hash_get (dirents, entry, APR_HASH_KEY_STRING))
+      else if (dirent_p)
         {
-          svn_node_kind_t kind;
-          fullpath = svn_path_join (path, entry, subpool);
-          SVN_ERR (svn_io_check_path (path, &kind, subpool));
-          SVN_ERR (send_unversioned_item (entry, kind, adm_access, 
+          SVN_ERR (send_unversioned_item (entry, dirent_p->kind,
+                                          dirent_p->special, adm_access, 
                                           patterns, eb->externals, no_ignore,
                                           eb->repos_locks, eb->repos_root,
                                           status_func, status_baton, subpool));
@@ -858,14 +896,14 @@ get_dir_status (struct edit_baton *eb,
       const void *key;
       apr_ssize_t klen;
       void *val;
-      svn_node_kind_t *path_kind;
+      svn_io_dirent_t *dirent_p;
 
       apr_hash_this (hi, &key, &klen, &val);
         
       /* Skip versioned things, and skip the administrative
          directory. */
-      if ((apr_hash_get (entries, key, klen)) 
-          || (strcmp (key, SVN_WC_ADM_DIR_NAME) == 0))
+      if (apr_hash_get (entries, key, klen)
+          || svn_wc_is_adm_dir (key, subpool))
         continue;
 
       /* Clear the iteration subpool. */
@@ -873,8 +911,9 @@ get_dir_status (struct edit_baton *eb,
 
       /* Make an unversioned status item for KEY, and put it into our
          return hash. */
-      path_kind = val;
-      SVN_ERR (send_unversioned_item (key, *path_kind, adm_access, 
+      dirent_p = val;
+      SVN_ERR (send_unversioned_item (key, dirent_p->kind, dirent_p->special,
+                                      adm_access, 
                                       patterns, eb->externals, no_ignore,
                                       eb->repos_locks, eb->repos_root,
                                       status_func, status_baton, iterpool));
@@ -883,7 +922,7 @@ get_dir_status (struct edit_baton *eb,
   /* Handle "this-dir" first. */
   if (! skip_this_dir)
     SVN_ERR (send_status_structure (path, adm_access, dir_entry, 
-                                    parent_entry, svn_node_dir,
+                                    parent_entry, svn_node_dir, FALSE,
                                     get_all, FALSE, eb->repos_locks,
                                     eb->repos_root, status_func, status_baton,
                                     subpool));
@@ -893,10 +932,13 @@ get_dir_status (struct edit_baton *eb,
     {
       const void *key;
       void *val;
+      svn_io_dirent_t *dirent_p;
 
-      /* Get the next dirent */
+      /* Get the next entry */
       apr_hash_this (hi, &key, NULL, &val);
 
+      dirent_p = apr_hash_get (dirents, key, APR_HASH_KEY_STRING);
+
       /* ### todo: What if the subdir is from another repository? */
           
       /* Skip "this-dir". */
@@ -907,7 +949,10 @@ get_dir_status (struct edit_baton *eb,
       svn_pool_clear (iterpool);
 
       /* Handle this directory entry (possibly recursing). */
-      SVN_ERR (handle_dir_entry (eb, adm_access, key, dir_entry, val, ignores, 
+      SVN_ERR (handle_dir_entry (eb, adm_access, key, dir_entry, val,
+                                 dirent_p ? dirent_p->kind : svn_node_none,
+                                 dirent_p ? dirent_p->special : FALSE,
+                                 ignores, 
                                  descend, get_all, no_ignore, 
                                  status_func, status_baton, cancel_func, 
                                  cancel_baton, iterpool));
@@ -939,14 +984,16 @@ hash_stash (void *baton,
 }
 
 
-/* Look up the key PATH in STATUSHASH.  If the value doesn't yet
-   exist, and the REPOS_TEXT_STATUS indicates that this is an
+/* Look up the key PATH in BATON->STATII.  IS_DIR_BATON indicates whether
+   baton is a struct *dir_baton or struct *file_baton.  If the value doesn't
+   yet exist, and the REPOS_TEXT_STATUS indicates that this is an
    addition, create a new status struct using the hash's pool.  Merge
    REPOS_TEXT_STATUS and REPOS_PROP_STATUS into the status structure's
    "network" fields.
    If a new struct was added, set the repos_lock to REPOS_LOCK. */
 static svn_error_t *
-tweak_statushash (apr_hash_t *statushash,
+tweak_statushash (void *baton,
+                  svn_boolean_t is_dir_baton,
                   svn_wc_adm_access_t *adm_access,
                   const char *path,
                   svn_boolean_t is_dir,
@@ -955,7 +1002,14 @@ tweak_statushash (apr_hash_t *statushash,
                   svn_lock_t *repos_lock)
 {
   svn_wc_status2_t *statstruct;
-  apr_pool_t *pool = apr_hash_pool_get (statushash);
+  apr_pool_t *pool;
+  apr_hash_t *statushash;
+
+  if (is_dir_baton)
+    statushash = ((struct dir_baton *) baton)->statii;
+  else
+    statushash = ((struct file_baton *) baton)->dir_baton->statii;
+  pool = apr_hash_pool_get (statushash);
 
   /* Is PATH already a hash-key? */
   statstruct = apr_hash_get (statushash, path, APR_HASH_KEY_STRING);
@@ -995,9 +1049,63 @@ tweak_statushash (apr_hash_t *statushash,
   if (repos_prop_status)
     statstruct->repos_prop_status = repos_prop_status;
   
+  /* Copy out of date info. */
+  if (is_dir_baton)
+    {
+      struct dir_baton *b = baton;
+      if (b->url)
+        statstruct->url = apr_pstrdup (pool, b->url);
+      statstruct->ood_kind = b->ood_kind;
+      /* The last committed rev, date, and author for deleted items
+         isn't available. */
+      if (statstruct->repos_text_status != svn_wc_status_deleted)
+        {
+          statstruct->ood_last_cmt_rev = b->ood_last_cmt_rev;
+          statstruct->ood_last_cmt_date = b->ood_last_cmt_date;
+          if (b->ood_last_cmt_author)
+            statstruct->ood_last_cmt_author =
+              apr_pstrdup (pool, b->ood_last_cmt_author);
+        }
+    }
+  else
+    {
+      struct file_baton *b = baton;
+      if (b->url)
+        statstruct->url = apr_pstrdup (pool, b->url);
+      statstruct->ood_last_cmt_rev = b->ood_last_cmt_rev;
+      statstruct->ood_last_cmt_date = b->ood_last_cmt_date;
+      statstruct->ood_kind = b->ood_kind;
+      if (b->ood_last_cmt_author)
+        statstruct->ood_last_cmt_author =
+          apr_pstrdup (pool, b->ood_last_cmt_author);
+    }
   return SVN_NO_ERROR;
 }
 
+/* Returns the URL for DB, or NULL: */
+static const char *
+find_dir_url (const struct dir_baton *db, apr_pool_t *pool)
+{
+  /* If we have no name, we're the root, return the anchor URL. */
+  if (! db->name)
+    return db->edit_baton->anchor_status->entry->url;
+  else
+    {
+      const char *url;
+      struct dir_baton *pb = db->parent_baton;
+      svn_wc_status2_t *status = apr_hash_get (pb->statii, db->name,
+                                               APR_HASH_KEY_STRING);
+      if (status && status->entry)
+        return status->entry->url;
+
+      url = find_dir_url (pb, pool);
+      if (url)
+        return svn_path_url_add_component (url, db->name, pool);
+      else
+        return NULL;
+    }
+}
+
 
 
 /* Create a new dir_baton for subdir PATH. */
@@ -1025,12 +1133,17 @@ make_dir_baton (void **dir_baton,
     full_path = apr_pstrdup (pool, eb->anchor);
 
   /* Finish populating the baton members. */
-  d->path         = full_path;
-  d->name         = path ? (svn_path_basename (path, pool)) : NULL;
-  d->edit_baton   = edit_baton;
+  d->path = full_path;
+  d->name = path ? (svn_path_basename (path, pool)) : NULL;
+  d->edit_baton = edit_baton;
   d->parent_baton = parent_baton;
-  d->pool         = pool;
-  d->statii       = apr_hash_make (pool);
+  d->pool = pool;
+  d->statii = apr_hash_make (pool);
+  d->url = apr_pstrdup (pool, find_dir_url (d, pool));
+  d->ood_last_cmt_rev = SVN_INVALID_REVNUM;
+  d->ood_last_cmt_date = 0;
+  d->ood_kind = svn_node_dir;
+  d->ood_last_cmt_author = NULL;
 
   /* Get the status for this path's children.  Of course, we only want
      to do this if the path is versioned as a directory. */
@@ -1042,13 +1155,14 @@ make_dir_baton (void **dir_baton,
   /* Order is important here.  We can't depend on parent_status->entry
      being non-NULL until after we've checked all the conditions that
      might indicate that the parent is unversioned ("unversioned" for
-     our purposes includes being an external). */
+     our purposes includes being an external or ignored item). */
   if (parent_status
       && (parent_status->text_status != svn_wc_status_unversioned)
       && (parent_status->text_status != svn_wc_status_deleted)
       && (parent_status->text_status != svn_wc_status_missing)
       && (parent_status->text_status != svn_wc_status_obstructed)
       && (parent_status->text_status != svn_wc_status_external)
+      && (parent_status->text_status != svn_wc_status_ignored)
       && (parent_status->entry->kind == svn_node_dir)
       && (eb->descend || (! pb)))
     {
@@ -1085,39 +1199,21 @@ make_file_baton (struct dir_baton *parent_dir_baton,
     full_path = apr_pstrdup (pool, eb->anchor);
 
   /* Finish populating the baton members. */
-  f->path       = full_path;
-  f->name       = svn_path_basename (path, pool);
-  f->pool       = pool;
-  f->dir_baton  = pb;
+  f->path = full_path;
+  f->name = svn_path_basename (path, pool);
+  f->pool = pool;
+  f->dir_baton = pb;
   f->edit_baton = eb;
-
+  f->url = svn_path_url_add_component (find_dir_url (pb, pool),
+                                       svn_path_basename (full_path, pool),
+                                       pool);
+  f->ood_last_cmt_rev = SVN_INVALID_REVNUM;
+  f->ood_last_cmt_date = 0;
+  f->ood_kind = svn_node_file;
+  f->ood_last_cmt_author = NULL;
   return f;
 }
 
-/* Returns the URL for DB, or NULL: */
-static const char *
-find_dir_url (const struct dir_baton *db, apr_pool_t *pool)
-{
-  /* If we have no name, we're the root, return the anchor URL. */
-  if (! db->name)
-    return db->edit_baton->anchor_status->entry->url;
-  else
-    {
-      const char *url;
-      struct dir_baton *pb = db->parent_baton;
-      svn_wc_status2_t *status = apr_hash_get (pb->statii, db->name,
-                                               APR_HASH_KEY_STRING);
-      if (status && status->entry)
-        return status->entry->url;
-
-      url = find_dir_url (pb, pool);
-      if (url)
-        return svn_path_url_add_component (url, db->name, pool);
-      else
-        return NULL;
-    }
-}
-
 /* Return a boolean answer to the question "Is STATUS something that
    should be reported?".  EB is the edit baton. */
 static svn_boolean_t 
@@ -1356,7 +1452,7 @@ delete_entry (const char *path,
 
   SVN_ERR (svn_wc_entries_read (&entries, adm_access, FALSE, pool));
   if (apr_hash_get (entries, hash_key, APR_HASH_KEY_STRING))
-    SVN_ERR (tweak_statushash (db->statii, eb->adm_access,
+    SVN_ERR (tweak_statushash (db, TRUE, eb->adm_access,
                                full_path, kind == svn_node_dir,
                                svn_wc_status_deleted, 0, NULL));
 
@@ -1364,7 +1460,7 @@ delete_entry (const char *path,
      is the root node and we're not supposed to report on the root
      node).  */
   if (db->parent_baton && (! *eb->target))
-    SVN_ERR (tweak_statushash (db->parent_baton->statii, eb->adm_access,
+    SVN_ERR (tweak_statushash (db->parent_baton, TRUE, eb->adm_access,
                                db->path, kind == svn_node_dir,
                                svn_wc_status_modified, 0, NULL));
 
@@ -1418,6 +1514,22 @@ change_dir_prop (void *dir_baton,
   struct dir_baton *db = dir_baton;
   if (svn_wc_is_normal_prop (name))    
     db->prop_changed = TRUE;
+
+  /* Note any changes to the repository. */
+  if (value != NULL)
+    {
+      if (strcmp (name, SVN_PROP_ENTRY_COMMITTED_REV) == 0)
+        db->ood_last_cmt_rev = SVN_STR_TO_REV (value->data);
+      else if (strcmp (name, SVN_PROP_ENTRY_LAST_AUTHOR) == 0)
+        db->ood_last_cmt_author = apr_pstrdup (db->pool, value->data);
+      else if (strcmp (name, SVN_PROP_ENTRY_COMMITTED_DATE) == 0)
+        {
+          apr_time_t tm;
+          SVN_ERR (svn_time_from_cstring (&tm, value->data, db->pool));
+          db->ood_last_cmt_date = tm;
+        }
+    }
+
   return SVN_NO_ERROR;
 }
 
@@ -1456,7 +1568,7 @@ close_directory (void *dir_baton,
       if (pb)
         /* NOTE: When we add directory locking, we need to find a directory
            lock here. */
-        SVN_ERR (tweak_statushash (pb->statii,
+        SVN_ERR (tweak_statushash (pb, TRUE,
                                    eb->adm_access,
                                    db->path, TRUE,
                                    repos_text_status,
@@ -1595,6 +1707,24 @@ change_file_prop (void *file_baton,
   struct file_baton *fb = file_baton;
   if (svn_wc_is_normal_prop (name))
     fb->prop_changed = TRUE;
+
+  /* Note any changes to the repository. */
+  if (value != NULL)
+    {
+      if (strcmp (name, SVN_PROP_ENTRY_COMMITTED_REV) == 0)
+        fb->ood_last_cmt_rev = SVN_STR_TO_REV (value->data);
+      else if (strcmp (name, SVN_PROP_ENTRY_LAST_AUTHOR) == 0)
+        fb->ood_last_cmt_author = apr_pstrdup (fb->dir_baton->pool,
+                                               value->data);
+      else if (strcmp (name, SVN_PROP_ENTRY_COMMITTED_DATE) == 0)
+        {
+          apr_time_t tm;
+          SVN_ERR (svn_time_from_cstring (&tm, value->data,
+                                          fb->dir_baton->pool));
+          fb->ood_last_cmt_date = tm;
+        }
+    }
+
   return SVN_NO_ERROR;
 }
 
@@ -1640,7 +1770,7 @@ close_file (void *file_baton,
       repos_prop_status = fb->prop_changed ? svn_wc_status_modified : 0;
     }
 
-  SVN_ERR (tweak_statushash (fb->dir_baton->statii,
+  SVN_ERR (tweak_statushash (fb, FALSE,
                              fb->edit_baton->adm_access,
                              fb->path, FALSE,
                              repos_text_status,
@@ -1657,6 +1787,7 @@ close_edit (void *edit_baton,
 {
   struct edit_baton *eb = edit_baton;
   apr_array_header_t *ignores = eb->ignores;
+  svn_error_t *err = NULL;
 
   /* If we get here and the root was not opened as part of the edit,
      we need to transmit statuses for everything.  Otherwise, we
@@ -1672,48 +1803,58 @@ close_edit (void *edit_baton,
       svn_node_kind_t kind;
       const char *full_path = svn_path_join (eb->anchor, eb->target, pool);
 
-      SVN_ERR (svn_io_check_path (full_path, &kind, pool));
+      err = svn_io_check_path (full_path, &kind, pool);
+      if (err) goto cleanup;
+
       if (kind == svn_node_dir)
         {
           svn_wc_adm_access_t *tgt_access;
           const svn_wc_entry_t *tgt_entry;
 
-          SVN_ERR (svn_wc_entry (&tgt_entry, full_path, eb->adm_access, 
-                                 FALSE, pool));
+          err = svn_wc_entry (&tgt_entry, full_path, eb->adm_access,
+                              FALSE, pool);
+          if (err) goto cleanup;
+
           if (! tgt_entry)
             {
-              SVN_ERR (get_dir_status (eb, NULL, eb->adm_access, eb->target, 
-                                       ignores, FALSE, eb->get_all, TRUE,
-                                       TRUE, eb->status_func, eb->status_baton,
-                                       eb->cancel_func, eb->cancel_baton,
-                                       pool));
+              err = get_dir_status (eb, NULL, eb->adm_access, eb->target, 
+                                    ignores, FALSE, eb->get_all, TRUE,
+                                    TRUE, eb->status_func, eb->status_baton,
+                                    eb->cancel_func, eb->cancel_baton,
+                                    pool);
+              if (err) goto cleanup;
             }
           else
             {
-              SVN_ERR (svn_wc_adm_retrieve (&tgt_access, eb->adm_access,
-                                            full_path, pool));
-              SVN_ERR (get_dir_status (eb, NULL, tgt_access, NULL, ignores, 
-                                       eb->descend, eb->get_all, 
-                                       eb->no_ignore, FALSE, 
-                                       eb->status_func, eb->status_baton, 
-                                       eb->cancel_func, eb->cancel_baton,
-                                       pool));
+              err = svn_wc_adm_retrieve (&tgt_access, eb->adm_access,
+                                         full_path, pool);
+              if (err) goto cleanup;
+
+              err = get_dir_status (eb, NULL, tgt_access, NULL, ignores, 
+                                    eb->descend, eb->get_all, 
+                                    eb->no_ignore, FALSE, 
+                                    eb->status_func, eb->status_baton, 
+                                    eb->cancel_func, eb->cancel_baton,
+                                    pool);
+              if (err) goto cleanup;
             }
         }
       else
         {
-          SVN_ERR (get_dir_status (eb, NULL, eb->adm_access, eb->target, 
-                                   ignores, FALSE, eb->get_all, TRUE,
-                                   TRUE, eb->status_func, eb->status_baton,
-                                   eb->cancel_func, eb->cancel_baton, pool));
+          err = get_dir_status (eb, NULL, eb->adm_access, eb->target, 
+                                ignores, FALSE, eb->get_all, TRUE,
+                                TRUE, eb->status_func, eb->status_baton,
+                                eb->cancel_func, eb->cancel_baton, pool);
+          if (err) goto cleanup;
         }
     }
   else
     {
-      SVN_ERR (get_dir_status (eb, NULL, eb->adm_access, NULL, ignores, 
-                               eb->descend, eb->get_all, eb->no_ignore, 
-                               FALSE, eb->status_func, eb->status_baton, 
-                               eb->cancel_func, eb->cancel_baton, pool));
+      err = get_dir_status (eb, NULL, eb->adm_access, NULL, ignores, 
+                            eb->descend, eb->get_all, eb->no_ignore, 
+                            FALSE, eb->status_func, eb->status_baton, 
+                            eb->cancel_func, eb->cancel_baton, pool);
+      if (err) goto cleanup;
     }
 
  cleanup:
@@ -1727,7 +1868,7 @@ close_edit (void *edit_baton,
                     eb->anchor, APR_HASH_KEY_STRING, NULL);
     }
   
-  return SVN_NO_ERROR;
+  return err;
 }
 
 
@@ -1920,7 +2061,8 @@ svn_wc_status2 (svn_wc_status2_t **status,
     }
 
   SVN_ERR (assemble_status (status, path, adm_access, entry, parent_entry,
-                            svn_node_unknown, TRUE, FALSE, NULL, NULL, pool));
+                            svn_node_unknown, FALSE, /* bogus */
+                            TRUE, FALSE, NULL, NULL, pool));
   return SVN_NO_ERROR;
 }
 
@@ -1956,6 +2098,13 @@ svn_wc_dup_status2 (svn_wc_status2_t *orig_stat,
   if (orig_stat->repos_lock)
     new_stat->repos_lock = svn_lock_dup (orig_stat->repos_lock, pool);
 
+  if (orig_stat->url)
+    new_stat->url = apr_pstrdup (pool, orig_stat->url);
+
+  if (orig_stat->ood_last_cmt_author)
+    new_stat->ood_last_cmt_author
+      = apr_pstrdup (pool, orig_stat->ood_last_cmt_author);
+
   /* Return the new hotness. */
   return new_stat;
 }
@@ -1977,3 +2126,18 @@ svn_wc_dup_status (svn_wc_status_t *orig_stat,
   /* Return the new hotness. */
   return new_stat;
 }
+
+svn_error_t *
+svn_wc_get_ignores (apr_array_header_t **patterns,
+                    apr_hash_t *config,
+                    svn_wc_adm_access_t *adm_access,
+                    apr_pool_t *pool)
+{
+  apr_array_header_t *default_ignores;
+
+  SVN_ERR (svn_wc_get_default_ignores (&default_ignores, config, pool));
+  SVN_ERR (collect_ignore_patterns (patterns, default_ignores, adm_access, 
+                                    pool));
+
+  return SVN_NO_ERROR;
+}
