@@ -2,10 +2,10 @@
 #
 # mailer.py: send email describing a commit
 #
-# $HeadURL: http://svn.collab.net/repos/svn/branches/1.2.x/tools/hook-scripts/mailer/mailer.py $
-# $LastChangedDate: 2005-04-21 14:12:41 -0400 (Thu, 21 Apr 2005) $
-# $LastChangedBy: breser $
-# $LastChangedRevision: 14358 $
+# $HeadURL: http://svn.collab.net/repos/svn/branches/1.3.x/tools/hook-scripts/mailer/mailer.py $
+# $LastChangedDate: 2005-12-15 01:57:49 +0100 (Thu, 15 Dec 2005) $
+# $LastChangedBy: maxb $
+# $LastChangedRevision: 17790 $
 #
 # USAGE: mailer.py commit      REPOS REVISION [CONFIG-FILE]
 #        mailer.py propchange  REPOS REVISION AUTHOR PROPNAME [CONFIG-FILE]
@@ -36,6 +36,7 @@ import smtplib
 import re
 import tempfile
 import types
+import urllib
 
 import svn.fs
 import svn.delta
@@ -66,6 +67,8 @@ def main(pool, cmd, config_fname, repos_dir, cmd_args):
   elif cmd == 'lock' or cmd == 'unlock':
     author = cmd_args[0]
     repos = Repository(repos_dir, 0, pool) ### any old revision will do
+    # Override the repos revision author with the author of the lock/unlock
+    repos.author = author
     cfg = Config(config_fname, repos, { 'author' : author })
     messenger = Lock(pool, cfg, repos, author, cmd == 'lock')
   else:
@@ -132,7 +135,7 @@ class OutputBase:
 
     try:
       truncate_subject = int(
-          self.cfg.get('truncate-subject', group, params))
+          self.cfg.get('truncate_subject', group, params))
     except ValueError:
       truncate_subject = 0
 
@@ -189,6 +192,11 @@ class MailedOutput(OutputBase):
 
   def mail_headers(self, group, params):
     subject = self.make_subject(group, params)
+    try:
+      subject.encode('ascii')
+    except UnicodeError:
+      from email.Header import Header
+      subject = Header(subject, 'utf-8').encode()
     hdrs = 'From: %s\n'    \
            'To: %s\n'      \
            'Subject: %s\n' \
@@ -440,8 +448,9 @@ def get_commondir(dirlist):
     commondir = ''
     newdirs = dirlist
   else:
-    common = string.split(dirlist.pop(), '/')
-    for d in dirlist:
+    common = string.split(dirlist[0], '/')
+    for j in range(1, len(dirlist)):
+      d = dirlist[j]
       parts = string.split(d, '/')
       for i in range(len(common)):
         if i == len(parts) or common[i] != parts[i]:
@@ -561,6 +570,14 @@ class DiffSelections:
         self.add = False
 
 
+class DiffURLSelections:
+  def __init__(self, cfg, group):
+    self.add = cfg.get('diff_add_url', group, None)
+    self.copy = cfg.get('diff_copy_url', group, None)
+    self.delete = cfg.get('diff_delete_url', group, None)
+    self.modify = cfg.get('diff_modify_url', group, None)
+
+
 def generate_content(renderer, cfg, repos, changelist, group, params, paths,
                      pool):
 
@@ -569,6 +586,7 @@ def generate_content(renderer, cfg, repos, changelist, group, params, paths,
   date = time.ctime(svn.core.secs_from_timestr(svndate, pool))
 
   diffsels = DiffSelections(cfg, group, params)
+  diffurls = DiffURLSelections(cfg, group)
 
   show_nonmatching_paths = cfg.get('show_nonmatching_paths', group, params) \
       or 'yes'
@@ -582,7 +600,7 @@ def generate_content(renderer, cfg, repos, changelist, group, params, paths,
 
   if len(paths) != len(changelist) and show_nonmatching_paths == 'yes':
     other_diffs = DiffGenerator(changelist, paths, False, cfg, repos, date,
-                                group, params, diffsels, pool),
+                                group, params, diffsels, diffurls, pool),
   else:
     other_diffs = [ ]
 
@@ -594,11 +612,12 @@ def generate_content(renderer, cfg, repos, changelist, group, params, paths,
     added_data=generate_list('A', changelist, paths, True),
     removed_data=generate_list('R', changelist, paths, True),
     modified_data=generate_list('M', changelist, paths, True),
+    show_nonmatching_paths=show_nonmatching_paths,
     other_added_data=other_added_data,
     other_removed_data=other_removed_data,
     other_modified_data=other_modified_data,
     diffs=DiffGenerator(changelist, paths, True, cfg, repos, date, group,
-                        params, diffsels, pool),
+                        params, diffsels, diffurls, pool),
     other_diffs=other_diffs,
     )
   renderer.render(data)
@@ -633,7 +652,7 @@ class DiffGenerator:
   "This is a generator-like object returning DiffContent objects."
 
   def __init__(self, changelist, paths, in_paths, cfg, repos, date, group,
-               params, diffsels, pool):
+               params, diffsels, diffurls, pool):
     self.changelist = changelist
     self.paths = paths
     self.in_paths = in_paths
@@ -643,6 +662,7 @@ class DiffGenerator:
     self.group = group
     self.params = params
     self.diffsels = diffsels
+    self.diffurls = diffurls
     self.pool = pool
 
     self.idx = 0
@@ -651,6 +671,17 @@ class DiffGenerator:
     # we always have some items
     return True
 
+  def _gen_url(self, urlstr, repos_rev, change):
+    if not len(urlstr):
+      return None
+    args = {
+      'path' : change.path and urllib.quote(change.path) or None,
+      'base_path' : change.base_path and urllib.quote(change.base_path) or None,
+      'rev' : repos_rev,
+      'base_rev' : change.base_rev,
+      }
+    return urlstr % args
+    
   def __getitem__(self, idx):
     while 1:
       if self.idx == len(self.changelist):
@@ -659,6 +690,16 @@ class DiffGenerator:
       path, change = self.changelist[self.idx]
       self.idx = self.idx + 1
 
+      diff = diff_url = None
+      kind = None
+      label1 = None
+      label2 = None
+      src_fname = None
+      dst_fname = None
+      binary = None
+      singular = None
+      content = None
+      
       # just skip directories. they have no diffs.
       if change.item_kind == svn.core.svn_node_dir:
         continue
@@ -670,24 +711,76 @@ class DiffGenerator:
       # figure out if/how to generate a diff
 
       if not change.path:
-        # it was deleted. should we show deletion diffs?
-        if not self.diffsels.delete:
-          continue
+        # it was delete.
+        kind = 'D'
 
-        kind = 'R'
-        diff = svn.fs.FileDiff(self.repos.get_root(change.base_rev),
-                               change.base_path, None, None, self.pool)
+        # show the diff url?
+        if self.diffurls.delete:
+          diff_url = self._gen_url(self.diffurls.delete,
+                                   self.repos.rev, change)
 
-        label1 = '%s\t%s' % (change.base_path, self.date)
-        label2 = '(empty file)'
-        singular = True
+        # show the diff?
+        if self.diffsels.delete:
+          diff = svn.fs.FileDiff(self.repos.get_root(change.base_rev),
+                                 change.base_path, None, None, self.pool)
+
+          label1 = '%s\t%s' % (change.base_path, self.date)
+          label2 = '(empty file)'
+          singular = True
+          
       elif change.added:
         if change.base_path and (change.base_rev != -1):
-          # this file was copied. any diff to show? should we?
-          if not change.text_changed or not self.diffsels.copy:
-            continue
-
+          # this file was copied.
           kind = 'C'
+
+          # any diff of interest?
+          if change.text_changed:
+
+            # show the diff url?
+            if self.diffurls.copy:
+              diff_url = self._gen_url(self.diffurls.copy,
+                                       self.repos.rev, change)
+
+            # show the diff?
+            if self.diffsels.copy:
+              diff = svn.fs.FileDiff(self.repos.get_root(change.base_rev),
+                                     change.base_path,
+                                     self.repos.root_this, change.path,
+                                     self.pool)
+              label1 = change.base_path + '\t(original)'
+              label2 = '%s\t%s' % (change.path, self.date)
+              singular = False
+        else:
+          # the file was added.
+          kind = 'A'
+
+          # show the diff url?
+          if self.diffurls.add:
+            diff_url = self._gen_url(self.diffurls.add,
+                                     self.repos.rev, change)
+
+          # show the diff?
+          if self.diffsels.add:
+            diff = svn.fs.FileDiff(None, None, self.repos.root_this,
+                                   change.path, self.pool)
+            label1 = '(empty file)'
+            label2 = '%s\t%s' % (change.path, self.date)
+            singular = True
+
+      elif not change.text_changed:
+        # the text didn't change, so nothing to show.
+        continue
+      else:
+        # a simple modification.
+        kind = 'M'
+
+        # show the diff url?
+        if self.diffurls.modify:
+          diff_url = self._gen_url(self.diffurls.modify,
+                                   self.repos.rev, change)
+
+        # show the diff?
+        if self.diffsels.modify:
           diff = svn.fs.FileDiff(self.repos.get_root(change.base_rev),
                                  change.base_path,
                                  self.repos.root_this, change.path,
@@ -695,53 +788,28 @@ class DiffGenerator:
           label1 = change.base_path + '\t(original)'
           label2 = '%s\t%s' % (change.path, self.date)
           singular = False
+
+      if diff:
+        binary = diff.either_binary()
+        if binary:
+          content = src_fname = dst_fname = None
         else:
-          # the file was added. should we show it?
-          if not self.diffsels.add:
-            continue
-
-          kind = 'A'
-          diff = svn.fs.FileDiff(None, None, self.repos.root_this,
-                                 change.path, self.pool)
-          label1 = '(empty file)'
-          label2 = '%s\t%s' % (change.path, self.date)
-          singular = True
-
-      elif not change.text_changed:
-        # the text didn't change, so nothing to show.
-        continue
-      else:
-        # a simple modification. show the diff?
-        if not self.diffsels.modify:
-          continue
-
-        kind = 'M'
-        diff = svn.fs.FileDiff(self.repos.get_root(change.base_rev),
-                               change.base_path,
-                               self.repos.root_this, change.path,
-                               self.pool)
-        label1 = change.base_path + '\t(original)'
-        label2 = '%s\t%s' % (change.path, self.date)
-        singular = False
-
-      binary = diff.either_binary()
-      if binary:
-        content = src_fname = dst_fname = None
-      else:
-        src_fname, dst_fname = diff.get_files()
-        content = DiffContent(self.cfg.get_diff_cmd(self.group, {
-          'label_from' : label1,
-          'label_to' : label2,
-          'from' : src_fname,
-          'to' : dst_fname,
-          }))
+          src_fname, dst_fname = diff.get_files()
+          content = DiffContent(self.cfg.get_diff_cmd(self.group, {
+            'label_from' : label1,
+            'label_to' : label2,
+            'from' : src_fname,
+            'to' : dst_fname,
+            }))
 
       # return a data item for this diff
       return _data(
-        kind=kind,
         path=change.path,
         base_path=change.base_path,
         base_rev=change.base_rev,
+        diff=diff,
+        diff_url=diff_url,
+        kind=kind,
         label_from=label1,
         label_to=label2,
         from_fname=src_fname,
@@ -749,7 +817,6 @@ class DiffGenerator:
         binary=binary,
         singular=singular,
         content=content,
-        diff=diff,
         )
 
 
@@ -873,17 +940,30 @@ class TextCommitRenderer:
     w = self.output.write
 
     for diff in diffs:
+      if not diff.diff and not diff.diff_url:
+        continue
       if diff.kind == 'D':
         w('\nDeleted: %s\n' % diff.base_path)
+        if diff.diff_url:
+          w('Url: %s\n' % diff.diff_url)
       elif diff.kind == 'C':
         w('\nCopied: %s (from r%d, %s)\n'
           % (diff.path, diff.base_rev, diff.base_path))
+        if diff.diff_url:
+          w('Url: %s\n' % diff.diff_url)
       elif diff.kind == 'A':
         w('\nAdded: %s\n' % diff.path)
+        if diff.diff_url:
+          w('Url: %s\n' % diff.diff_url)
       else:
         # kind == 'M'
         w('\nModified: %s\n' % diff.path)
+        if diff.diff_url:
+          w('Url: %s\n' % diff.diff_url)
 
+      if not diff.diff:
+        continue
+      
       w(SEPARATOR + '\n')
 
       if diff.binary:
@@ -1014,6 +1094,8 @@ class Config:
   def _prep_maps(self):
     "Rewrite the [maps] options into callables that look up values."
 
+    mapsections = []
+
     for optname, mapvalue in vars(self.maps).items():
       if mapvalue[:1] == '[':
         # a section is acting as a mapping
@@ -1027,8 +1109,9 @@ class Config:
                 lambda value,
                        sect=getattr(self, sectname): getattr(sect, value,
                                                              value))
-        # remove the mapping section from consideration as a group
-        self._groups.remove(sectname)
+        # mark for removal when all optnames are done
+        if sectname not in mapsections:
+          mapsections.append(sectname)
 
       # elif test for other mapper types. possible examples:
       #   dbm:filename.db
@@ -1039,6 +1122,11 @@ class Config:
       else:
         raise UnknownMappingSpec(mapvalue)
 
+    # remove each mapping section from consideration as a group
+    for sectname in mapsections:
+      self._groups.remove(sectname)
+
+
   def _prep_groups(self, repos):
     self._group_re = [ ]
 
@@ -1133,11 +1221,11 @@ except NameError:
 
 if __name__ == '__main__':
   def usage():
+    scriptname = os.path.basename(sys.argv[0])
     sys.stderr.write(
 """USAGE: %s commit      REPOS REVISION [CONFIG-FILE]
        %s propchange  REPOS REVISION AUTHOR PROPNAME [CONFIG-FILE]
-       %s propchange2 REPOS REVISION AUTHOR PROPNAME ACTION
-       %s             [CONFIG-FILE]
+       %s propchange2 REPOS REVISION AUTHOR PROPNAME ACTION [CONFIG-FILE]
        %s lock        REPOS AUTHOR [CONFIG-FILE]
        %s unlock      REPOS AUTHOR [CONFIG-FILE]
 
@@ -1149,8 +1237,7 @@ ACTION was added as a fifth argument to the post-revprop-change hook
 in Subversion 1.2.0.  Its value is one of 'A', 'M' or 'D' to indicate
 if the property was added, modified or deleted, respectively.
 
-""" % (sys.argv[0], sys.argv[0], sys.argv[0], ' ' * len(sys.argv[0]),
-       sys.argv[0], sys.argv[0]))
+""" % (scriptname, scriptname, scriptname, scriptname, scriptname))
     sys.exit(1)
 
   # Command list:  subcommand -> number of arguments expected (not including
@@ -1190,7 +1277,6 @@ if the property was added, modified or deleted, respectively.
       config_fname = os.path.join(os.path.dirname(sys.argv[0]), 'mailer.conf')
   if not os.path.exists(config_fname):
     raise MissingConfig(config_fname)
-  config_fp = open(config_fname)
 
   svn.core.run_app(main, cmd, config_fname, repos_dir,
                    sys.argv[3:3+expected_args])
@@ -1208,9 +1294,7 @@ if the property was added, modified or deleted, respectively.
 #     o max size of entire commit message before truncation
 #   - per-repository configuration
 #     o extra config living in repos
-#     o how to construct a ViewCVS URL for the diff  [DONE (as patch)]
 #     o optional, non-mail log file
 #     o look up authors (username -> email; for the From: header) in a
 #       file(s) or DBM
-#   - if the subject line gets too long, then trim it. configurable?
 # * get rid of global functions that should properly be class methods
